/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : Dec. 2006

 File          : ADuC7038_IDD.c

 Hardware      : Applicable to ADuC703x

 Description   : Wake-up Timer causes part to power-up every 8s.
 				 the timer is configured in Hr:Min:Sec:Hundredths format
				
				Additional (Gilbert): Can be woke up by LIN during power down mode.

				 T2LD:
				 bits 31-24 represent the hours
				 bits 21-16 represent the minutes
				 bits 13-8 represent the secondes
				 bits 6-0 represent the hundredths

				 The debugger cannot be used when the part is placed in power down mode.
		
*********************************************************************/

#include <ADuC7038.h>

#define GPIO0  0x010000
#define GPIO1  0x020000
#define GPIO2  0x04000
#define GPIO3  0x080000
#define GPIO4  0x100000
#define GPIO5  0x200000

void My_FIQ_Function(void);
void My_IRQ_Function(void);
void LIN_INIT(void);
void delay(int);
char HVstatus;
int iFIQ_Source,itemp1,itemp2,itemp3, iIRQSIG;
	unsigned long it1=1234, it2=4321;


int main()
{
 	GPCON = 0x00000000;
	GPDAT = 0x3F3f0000;		// Set all bits to outputs
 // 	GPCLR = GPIO0;		//	Keep this low as ISOLATOR input/outputs need to be low for PD

	LIN_INIT();

	IRQ = My_IRQ_Function; 	// Initialise Interrupt Routine	 for LIN
	IRQEN = 0x08;
 	FIQ = My_FIQ_Function; 	// Initialise Interrupt Routine	 for LIN
	FIQEN = 0x20;

	T1LD = 0x1F40;   		// full scale 524 sec	/16 = 2 sec	  - lpck
	T1CON = 0x0D;		// 16 prescaler .. 2 sec time, low p clk

	T2LD = 0x00ff;		 // greater then 100ms - for test
	T2CON = 0x88;		// Watchdog on  - /256
		  // 0000 1010
  	GPCLR = GPIO5;	//	Take GPIO 5 low, turn on islator
//  	GPCLR = GPIO3;	//	wake up High side
  
  	while(1)
	{
	}				// Infinite Loop
}

void delay (int length)
{
	while (length >=0)
    	length--;
}

void My_IRQ_Function() 
{
 
  GPSET = GPIO5;	//	Take GPIO 5 High, turn off islator
  GPSET = GPIO3;	//	Take GPIO 5 High, turn off WAKE-Up
  delay(1000);
	iIRQSIG = IRQSIG;
	iFIQ_Source = IRQSTA;			// Save a temporary copy of IRQSTA		
		T1CLRI = 0x01;					// restart Timer 0
		POWKEY0 = 0x01;		 			// Put all peripherals, PLL and Core Asleep
		POWCON  = 0x00;					// default value is 79	- 10MHz
		POWKEY1 = 0xF4;					// 

		it1 = it1*it2;					// Dummy cycle to ensure correct power down

	return ;
}



void My_FIQ_Function() 
{
	int LIN_status;						// BLANK the part -emergency issue or module
//	while (FEESTA & 0x04);	// wait for the FLASH Controllor to be free.
//	FEEMOD = 0x08;
//	FEEADR = 0xFFC3;
//	FEEDAT = 0x3CFF;
//	FEECON = 0x06; 		//Mass Erase command
//	while (FEESTA & 0x04){} //Wait for command to finish

   	LIN_status = LINSTA;		// 0000 0011 1000 1001  - 389	- Frame error cleared
		
	if (LIN_status & 0x08)		// Break error has occurred.
		{
		GPCLR = GPIO3;		 
		delay(50);			 //  Togle for a whilt to Wake up.
		GPSET = GPIO3;		 // GPIO3 wakes High Side
		LINCON = 0x880;		 // Rx mode
		}
	 return;
}
void LIN_INIT(void )
{
 int  HVstatus;
	do										
	{
		HVDAT = 0x00; 						// Enable Normal LIN TX mode / reenable short circuit
		HVCON = 0x08; 						// Write to Config
		do
		{
			HVstatus = HVCON;
		}
		while(HVstatus & 0x1); 				//  command finished
	}
	while (!(HVstatus & 0x4));				// transmit command 

// LINWU = 0x0010;							// Wake Up with INWU also
// LINBK = 5500;						// Set number of 128kHz periods to generate a Break Interrupt
//LINBK = 0x7700;							// 0x46 / 128kHz ~ 492us which is just over 11 TBits.
											// According to LIN V2.0, a slave with a clock accuracy of <2% may
											// use a break detection of 9.5 bit, otherwise a break detection of
											// 11 bits is required
											// This is for 19200bps
	LINCON = 0x880;						// Enable LIN , checksum, BREAK=0 - enabled.
}
